//
//                     TxWin, Textmode Windowing Library
//
//   Original code Copyright (c) 1995-2021 Fsys Software and Jan van Wijk
//
// ==========================================================================
//
//   TxLib, released under MIT License
//
//   Copyright (c) 1995-2021  Fsys Software and Jan Van Wijk
//
//   Permission is hereby granted, free of charge, to any person obtaining a copy
//   of this software and associated documentation files (the "Software"), to deal
//   in the Software without restriction, including without limitation the rights
//   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//   copies of the Software, and to permit persons to whom the Software is
//   furnished to do so, subject to the following conditions:
//
//   The above copyright notice and this permission notice shall be included in all
//   copies or substantial portions of the Software.
//
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//   SOFTWARE.
//
//
//   Questions on TxWin licensing can be directed to: info@dfsee.com
//
// ==========================================================================
//
// TxLib filesystem functions, filename manipulations, base name
//
// JvW  28-02-2021 LICENSING: Changed from LGPL to the more liberal MIT license

#include <txlib.h>                              // TxLib interface


/*****************************************************************************/
// Get base-name part from a path+filename string
/*****************************************************************************/
char *TxGetBaseName                             // RET   ptr to basename
(
   char               *fname                    // IN    path+filename string
)
{
   char               *rc;

   if      ((rc = strrchr( fname, FS_PATH_SEP)) != NULL)
   {
      rc++;                                     // start of basename
   }
   else if ((rc = strrchr( fname, ':')) != NULL) // bare drive spec (x:file)
   {
      rc++;
   }
   else
   {
      rc = fname;                               // no path, use whole name
   }
   return( rc);
}                                               // end 'TxGetBaseName'
/*---------------------------------------------------------------------------*/


/*****************************************************************************/
// Strip basename from path+filename string, leaving the PATH component only
/*****************************************************************************/
char *TxStripBaseName                           // RET   BaseName location,
(                                               //       or NULL if not there
   char               *fname                    // IN    path+filename string
)                                               // OUT   path only string
{
   char               *rc = strrchr( fname, FS_PATH_SEP);

   if (rc != NULL)
   {
      *(rc++) = 0;                              // terminate at PATH_SEP loc
   }                                            // and advance to BaseName
   return( rc);
}                                               // end 'TxStripBaseName'
/*---------------------------------------------------------------------------*/

